"""pexpect compatibility shim — public module.

Use ``import ptylink.compat as pexpect`` for a drop-in replacement.
"""

from ptylink._compat import *  # noqa: F401, F403
from ptylink._compat import __all__  # noqa: F401
